@@ -53,10 +53,10 @@ private:
     float _video_aspect_ratio;
     int _video_frame_rate_num;
     int _video_frame_rate_den;
-    video_frame_format _video_preferred_frame_format;
+    enum decoder::video_frame_format _video_preferred_frame_format;
     int _audio_rate;
     int _audio_channels;
-    int _audio_bits;
+    enum decoder::audio_sample_format _audio_sample_format;
     int64_t _duration;
     blob _audio_buffer;
 
@@ -105,11 +105,16 @@ public:
         return static_cast<int64_t>(_video_frame_rate_den) * 1000000 / _video_frame_rate_num;
     }
 
-    video_frame_format video_preferred_frame_format() const throw ()
+    enum decoder::video_frame_format video_preferred_frame_format() const throw ()
     {
         return _video_preferred_frame_format;
     }
 
+    bool has_audio() const throw ()
+    {
+        return (_audio_rate > 0);
+    }
+
     int audio_rate() const throw ()
     {
         return _audio_rate;
@@ -120,9 +125,9 @@ public:
         return _audio_channels;
     }
 
-    int audio_bits() const throw ()
+    enum decoder::audio_sample_format audio_sample_format() const throw ()
     {
-        return _audio_bits;
+        return _audio_sample_format;
     }
 
     int64_t duration() const throw ()
@@ -131,7 +136,7 @@ public:
     }
 
     int64_t read_video_frame();
-    void get_video_frame(video_frame_format fmt,
+    void get_video_frame(enum decoder::video_frame_format fmt,
             uint8_t *l_data[3], size_t l_line_size[3],
             uint8_t *r_data[3], size_t r_line_size[3]);
     void release_video_frame();
